function k=BKernel(u,v,eta,sigma)

%% (c) 2004 Marco Cuturi , marco.cuturi@ensmp.fr
%% Distributed under GPL licence


%%%%%%% Implements Kernelized version of Bhattacharyya Kernel 
%%%%%%% As exposed in Risi Kondor and Tony Jebara, A Kernel between Sets of Vectors
%%%%%%% Proceedings of ICML 2003.

%%%%%%% This code is a modified version of the one proposed by Risi on his
%%%%%%% website.

%%%%%%% u : image as a line vector of  [L, d, (pixels), (0)] where L is a
%%%%%%% label, d is the number of non-zero pixels, the remaining space
%%%%%%% beging filled with zeros.
%%%%%%% v : same
%%%%%%% eta : regularization
%%%%%%% sigma : Gaussian kernel bandwith.

%%%%%%% Those first lines are just there to use the format used by the
%%%%%%% handwritten digits experiment.
X1=transpose(u(3:2+u(2)));
X2=transpose(v(3:2+v(2)));

[n1,dummy]=size(X1);
[n2,dummy]=size(X2);

n=n1+n2;
X=[X1;X2];
K=zeros(n,n);

%%%%%%%%% Gram Matrix Computation %%%%%%%%
for i=1:n
  for j=1:i
    K(i,j)=RBFB(X(i),X(j),sigma); 
    K(j,i)=K(i,j);                
  end;
end;

%%%%%%%%% Tiny regularization to perform generalized eigendecomposition below  
%%%%%%%%% to produce an orthogonal base. Not very quick or elegant but so far this
%%%%%%%%% looks more stable than straightforward Gram-Schmidt.
K=K+.0001*eye(n);
[M,R]=eig(K^2,K);

V=K*M;

V1=V(1:n1,:);   
V2=V(n1+1:n,:); 

m1=V1'*(1/n1)*ones(n1,1);  
m2=V2'*(1/n2)*ones(n2,1); 

S1=V1'*(1/n1*eye(n1)-1/n1^2*ones(n1))*V1+eta*eye(n);
S2=V2'*(1/n2*eye(n2)-1/n2^2*ones(n2))*V2+eta*eye(n);

S1inv=inv(S1);
S2inv=inv(S2);

mdag=1/2*(S1inv*m1+S2inv*m2);
Sdag=2*inv(S1inv+S2inv);

a=-m1'*S1inv*m1/4;
b=-m2'*S2inv*m2/4;
c=+mdag'*Sdag*mdag/2;

%%% We use -log of the value defined in Kondor & Jebara 2003.
k=-log(det(S1)^(-1/4)*det(S2)^(-1/4)*det(Sdag)^(1/2))-(a+b+c);